﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Collections.Generic;
using System.Windows.Shapes;

namespace TravelPlannerScreens
{
    public class TimelineDragBehavior : Behavior<FrameworkElement>
    {
        // ドラッグ中半透明フラグ
        private bool isGhost = true;

        // ドラッグ開始のエレメント位置
        private Point dragStartElementPoint;

        // ドラッグ中フラグ
        private bool mouseIsDragging = false;

        /// <summary>
        /// 半透明フラグ
        /// </summary>
        public bool IsGhost
        {
            get { return this.isGhost; }
            set { this.isGhost = value; }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            // Behavior がオブジェクトにアタッチされた時に実行したいコードを挿入します。
            this.AssociatedObject.MouseLeftButtonDown += new MouseButtonEventHandler(AssociatedObject_MouseLeftButtonDown);
            this.AssociatedObject.MouseLeftButtonUp += new MouseButtonEventHandler(AssociatedObject_MouseLeftButtonUp);
            this.AssociatedObject.MouseMove += new MouseEventHandler(AssociatedObject_MouseMove);
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            // Behavior がオブジェクトから削除された時に実行したいコードを挿入します。
            this.AssociatedObject.MouseLeftButtonDown -= new MouseButtonEventHandler(AssociatedObject_MouseLeftButtonDown);
            this.AssociatedObject.MouseLeftButtonUp -= new MouseButtonEventHandler(AssociatedObject_MouseLeftButtonUp);
            this.AssociatedObject.MouseMove -= new MouseEventHandler(AssociatedObject_MouseMove);
        }

        // マウスダウンイベントハンドラ
        void AssociatedObject_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            // ドラッグ開始
            this.AssociatedObject.CaptureMouse();
            this.mouseIsDragging = true;

            if (this.isGhost == true)
            {
                this.AssociatedObject.Opacity = 0.5;
            }
        }

        // マウスアップイベントハンドラ
        void AssociatedObject_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Panel ParentPanel = (Panel)this.AssociatedObject.Parent;
            Panel RootPanel = (Panel)ParentPanel.Parent;

            Point p = e.GetPosition(RootPanel);

            // 吸着処理
            if (p.Y > 405)
            {
                Canvas.SetTop(this.AssociatedObject, 296);
            }
            else if (p.Y > 260)
            {
                Canvas.SetTop(this.AssociatedObject, 150);
            }
            else
            {
                Canvas.SetTop(this.AssociatedObject, -1);
            }

            if(p.X < 241)
            {
                Canvas.SetLeft(this.AssociatedObject, 172);
            }
            else if (p.X > 728)
            {
                Canvas.SetLeft(this.AssociatedObject, 655);
            }

            // ドラッグ終了
            this.mouseIsDragging = false;
            this.AssociatedObject.ReleaseMouseCapture();

            if (this.isGhost == true)
            {
                this.AssociatedObject.Opacity = 1;
            }
        }

        // マウスムーブイベントハンドラ
        void AssociatedObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.mouseIsDragging)
            {
                Point movePoint = e.GetPosition((UIElement)this.AssociatedObject.Parent);

                this.AssociatedObject.SetValue(Canvas.LeftProperty, (movePoint.X - (this.AssociatedObject.Width/2)));
                this.AssociatedObject.SetValue(Canvas.TopProperty, (movePoint.Y - (this.AssociatedObject.Height/2)));

                Canvas parentCanvas = this.AssociatedObject.Parent as Canvas;

                foreach (UIElement item in parentCanvas.Children)
                {
                    Canvas.SetZIndex(item, 0);
                }

                Canvas.SetZIndex(this.AssociatedObject, 100);
            }
        }
    }
}